@@ -25,6 +25,10 @@
 /* include the names for the caps and a definition of __CAP_BITS */
 #include "cap_names.h"
 
+#ifndef _LINUX_CAPABILITY_U32S_1
+# define _LINUX_CAPABILITY_U32S_1          1
+#endif /* ndef _LINUX_CAPABILITY_U32S */
+
 /*
  * Do we match the local kernel?
  */
@@ -34,7 +38,7 @@
 # error Kernel <linux/capability.h> does not support library
 # error file "libcap.h" --> fix and recompile libcap
 
-#elif (_LINUX_CAPABILITY_VERSION == 0x19980330)
+#elif !defined(_LINUX_CAPABILITY_VERSION_2)
 
 # warning Kernel <linux/capability.h> does not support 64-bit capabilities
 # warning and libcap is being built with no support for 64-bit capabilities
@@ -43,13 +47,34 @@
 #  define _LINUX_CAPABILITY_VERSION_1 0x19980330
 # endif
 
-#elif (_LINUX_CAPABILITY_VERSION != 0x20071026)
+# _LIBCAP_CAPABILITY_VERSION  _LINUX_CAPABILITY_VERSION_1
+# _LIBCAP_CAPABILITY_U32S     _LINUX_CAPABILITY_U32S_1
+
+#elif defined(_LINUX_CAPABILITY_VERSION_3)
+
+# if (_LINUX_CAPABILITY_VERSION_3 != 0x20080522)
+#  error Kernel <linux/capability.h> v3 does not match library
+#  error file "libcap.h" --> fix and recompile libcap
+# else
+#  define _LIBCAP_CAPABILITY_VERSION  _LINUX_CAPABILITY_VERSION_3
+#  define _LIBCAP_CAPABILITY_U32S     _LINUX_CAPABILITY_U32S_3
+# endif
+
+#elif (_LINUX_CAPABILITY_VERSION_2 != 0x20071026)
 
 # error Kernel <linux/capability.h> does not match library
 # error file "libcap.h" --> fix and recompile libcap
 
+#else
+
+# define _LIBCAP_CAPABILITY_VERSION  _LINUX_CAPABILITY_VERSION_2
+# define _LIBCAP_CAPABILITY_U32S     _LINUX_CAPABILITY_U32S_2
+
 #endif
 
+#undef _LINUX_CAPABILITY_VERSION
+#undef _LINUX_CAPABILITY_U32S
+
 /*
  * This is a pointer to a struct containing three consecutive
  * capability sets in the order of the cap_flag_t type: the are
@@ -59,10 +84,6 @@
  * to processes.
  */
 
-#ifndef _LINUX_CAPABILITY_U32S
-# define _LINUX_CAPABILITY_U32S          1
-#endif /* ndef _LINUX_CAPABILITY_U32S */
-
 #if defined(VFS_CAP_REVISION_MASK) && !defined(VFS_CAP_U32)
 # define VFS_CAP_U32_1                   1
 # define XATTR_CAPS_SZ_1                 (sizeof(__le32)*(1 + 2*VFS_CAP_U32_1))
@@ -86,7 +107,7 @@ struct _cap_vfs_cap_data {
 #endif /* ndef CAP_TO_MASK */
 
 #define NUMBER_OF_CAP_SETS      3   /* effective, inheritable, permitted */
-#define __CAP_BLKS   (_LINUX_CAPABILITY_U32S)
+#define __CAP_BLKS   (_LIBCAP_CAPABILITY_U32S)
 #define CAP_SET_SIZE (__CAP_BLKS * sizeof(__u32))
 
 #define CAP_T_MAGIC 0xCA90D0
@@ -95,9 +116,12 @@ struct _cap_struct {
     union {
 	struct __user_cap_data_struct set;
 	__u32 flat[NUMBER_OF_CAP_SETS];
-    } u[_LINUX_CAPABILITY_U32S];
+    } u[_LIBCAP_CAPABILITY_U32S];
 };
 
+/* the maximum bits supportable */
+#define __CAP_MAXBITS (__CAP_BLKS * 32)
+
 /* string magic for cap_free */
 #define CAP_S_MAGIC 0xCA95D0
 
@@ -132,7 +156,7 @@ struct _cap_struct {
 # define _cap_debugcap(s, c, set) do { \
     unsigned _cap_index; \
     fprintf(stderr, __FUNCTION__ "(" __FILE__ ":%d): " s, __LINE__); \
-    for (_cap_index=_LINUX_CAPABILITY_U32S; _cap_index-- > 0; ) { \
+    for (_cap_index=_LIBCAP_CAPABILITY_U32S; _cap_index-- > 0; ) { \
        fprintf(stderr, "%08x", (c).u[_cap_index].flat[set]); \
     } \
     fprintf(stderr, "\n"); \
@@ -163,7 +187,7 @@ extern int capsetp(pid_t pid, cap_t cap_d);
 #define PR_SET_KEEPCAPS    8
 #define PR_CAPBSET_READ   23
 #define PR_CAPBSET_DROP   24
-#define PR_GET_SECUREBITS 25
-#define PR_SET_SECUREBITS 26
+#define PR_GET_SECUREBITS 27
+#define PR_SET_SECUREBITS 28
 
 #endif /* LIBCAP_H */
